import Variables
import pygame
def AddObject(image):
  Variables.parts.append(image)
  index = len(Variables.parts) - 1
  Variables.parts[index].index = index
  return index
def AddObjects(images):
  indexes = []
  for i in images:
    indexes.append(AddObject(i))
  return indexes

def AddPrefab(name, obj):
  newObj = obj.Copy()
  newObj.Destroy() #need to remove prefab from child list
  newObj.destroyed = False #so prefab isnt still destroyed
  newObj.parent = None
  Variables.prefabs[name] = newObj
def GetPrefab(name):
  newObj = Variables.prefabs[name].Copy()
  newObj.Move(len(newObj.GetParentChildren()))
  return newObj
def PrintFonts():
  fonts = pygame.font.get_fonts() 
  for i in fonts:
    print(i)




import pygame
import copy 
import Classes
import Variables
import Functions
from datetime import datetime
def init():
  pygame.init()
  #Screen######
  width, height = 800,  600
  global backgroundColor
  global screen
  backgroundColor = 255, 255, 255
  screen = pygame.display.set_mode((width, height))
  #############

  #FUNCTIONS##################
  def Touching(rect, pos):
    return rect.left+rect.width > pos[0] > rect.left and rect.top+rect.height > pos[1] > rect.top
  ############################
  for i in Variables.starts:
    i()#run start
  ###VARIABLES##########
  global mouse
  mouse = (0, 0)
  mousePressed = False
  mouseDragging = False
  #########################################

  #MAIN FUNCTION###########################
  def MainFunction(i, offset):
    #SET RECT##############################
      rect = None
      if(not(i.text is None)):
        font = pygame.font.Font(i.text.font + ".ttf", i.text.fontSize) 
        text = font.render(i.text.text, True, i.text.fontColor, i.text.backgroundColor)
        rect = font.size((i.text.text))
        rect = pygame.Rect(i.rect[0], i.rect[1], rect[0], rect[1])
        rect.left = rect.left + offset[0]
        rect.top = rect.top + offset[1]
        #print(rect)
        screen.blit(text, rect)
      elif(not(i.image is None)):
        rect = copy.deepcopy(i.rect)
        rect.left = rect.left + offset[0]
        rect.top = rect.top + offset[1]
        screen.blit(i.image, rect)
      else:
        rect = copy.deepcopy(i.rect)
        rect.left = rect.left + offset[0]
        rect.top = rect.top + offset[1]
        pygame.draw.rect(screen, i.color, rect)
      ########################################
      #RUN BUTTON PRESSES#####################
      if i.button != None:
        hovering = Touching(rect, mouse)
        touching = mousePressed and hovering
        if(touching):
          if(i.button.onClick != None):
            i.button.onClick(i)
          i.button.dragging = True
        elif(i.button.dragging and (hovering or i.button.enableDragOff) and mouseDragging):
          if(i.button.onDrag != None):
            i.button.onDrag(i)
        elif((not hovering and not i.button.enableDragOff) or not mouseDragging):
          if(i.button.onClickOff != None and i.button.dragging == True):
            i.button.onClickOff(i)
          i.button.dragging = False
      ########################################

      #RUN COMPONENT Updates#########################
      for iComp in i.components:
        if(iComp[1] != None):
          iComp[1](i)
      for iChild in i.children:
        MainFunction(iChild, (rect.left, rect.top))
      ########################################
  #########################################

  while True:
    startTime = datetime.now()#Set Start Time
    #GetMousePos##############################
    mousePressed = False
    for event in pygame.event.get():
      if event.type == pygame.MOUSEBUTTONDOWN: 
        mouse = pygame.mouse.get_pos()
        mousePressed = True;
        mouseDragging = True
      elif event.type == pygame.MOUSEBUTTONUP:
        mouseDragging = False
    ##########################################
    screen.fill(backgroundColor)
    for i in Variables.parts:
      MainFunction(i, (0, 0))
    Variables.deltaTime = (datetime.now()-startTime).total_seconds()#Set DeltaTime
    pygame.display.flip()
  #################


import Variables
import copy
import pygame
class Object:
  def __init__(self, rect, image, onClick = None, onDrag = None, onClickOff = None, enableDragOff = False):
    self.destroyed = False
    self.parent = None
    self.children = []
    self.image = None
    self.text = None
    self.color = None
    self.components = []
    if(type(image) is tuple):
      self.color = image
    elif(type(image) is String):
      self.text = image
    else:
      self.image = image 
      ratio = image.get_size()
      min = ratio[0]
      if(min > ratio[1]):
        min = ratio[1]
      ratio = (int(rect.width / ratio[1] * min), int(rect.height / ratio[0] * min))
      print(ratio)
      self.image = pygame.transform.scale(self.image, ratio)
    self.rect = rect
    if onClick != None or onDrag != None:
      self.button = Button(onClick, onDrag, onClickOff, enableDragOff)
    else:
      self.button = None
  #####
  def Decendants(self):
    Throw(self)
    indexes = []
    for i in self.children:
      indexes.append(i)
      indexes.extend(Variables.parts[i].Decendants())
    return indexes
  def Move(self, newIndex):#components can have their update function skipped on accident if ran in another comp.update
    Throw(self)
    children = self.GetParentChildren()
    oldIndex = self.index
    self.index = newIndex
    del children[oldIndex]
    children.insert(newIndex, self)
    i = oldIndex
    if(newIndex > oldIndex):
      while(i < newIndex):
        children[i].index = i
        i = i + 1
    elif(newIndex < oldIndex):
      while(i >= newIndex):
        children[i].index = i
        i = i - 1
  def SetParent(self, newParent):
    Throw(self)
    children = self.GetParentChildren()
    self.Move(len(children))#make it so other children have their indexes fixed
    del children[self.index]
    self.parent = newParent
    self.parent.children.append(self)
    self.index = len(self.parent.children) - 1
  def GetParentChildren(self):
    Throw(self)
    children = None
    if(self.parent != None):
      children = self.parent.children
    else:
      children = Variables.parts
    return children
  def Copy(self):
    Throw(self)
    CopyHelp(self)

    parChildren = self.GetParentChildren()
    copied = copy.deepcopy(self)
    
    copied.index = len(parChildren) - 1
    parChildren.append(copied)
    copied.Move(self.index + 1)
    CopyUnHelp(self)
    CopyUnHelp(copied)
    return copied
  def Destroy(self):
    Throw(self)
    children = self.GetParentChildren()
    self.Move(len(children))
    del children[self.index]
    self.index = -1 #will cause error if used incorrectly, did this on purpous
    self.destroyed = True
    for i in self.children:
      i.Destroy()
  def AddComp(self, comp):
    self.components.append(comp)
    if(comp[0] != None):
        comp[0](self)
class Button:
  def __init__(self, onClick, onDrag, onClickOff, enableDragOff):
    self.onClick = onClick
    self.onClickOff = onClickOff
    self.onDrag = onDrag
    self.dragging = False
    self.enableDragOff = enableDragOff
class String:
  def __init__(self, text, fontSize, font, fontColor, backgroundColor):
    self.fontSize = fontSize
    self.text = text
    self.font = font
    self.fontColor = fontColor
    self.backgroundColor = backgroundColor

def CopyHelp(self):
  if(self.image != None):
    self.image = pygame.image.tostring(self.image, 'RGBA')
  for i in self.children:
    CopyHelp(i)

def CopyUnHelp(self):
  if(self.image != None):
    self.image = pygame.image.fromstring(self.image, self.rect.size, 'RGBA')
  for i in self.children:
    CopyUnHelp(i)
def Throw(self):
  if(self.destroyed):
    raise Exception("Object was destroyed")